//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Threading;
using System.Runtime.InteropServices;
using System.Security;
using System.Security.Permissions;
using System.IO;
using System.Text;

namespace Exceptions
{
	class Recurse
	{
		public static void StackOverflow()
		{
			Recurse.StackOverflow();
		}
	}
	// Custom exception class derived from ApplicationException
	class Custom1Exception : ApplicationException
	{
		public Custom1Exception(String msg) : base(msg)
		{
		}
	}
	class Custom2Exception : ApplicationException
	{
		public Custom2Exception(String msg) : base(msg)
		{
		}
	}
	class Custom3Exception : ApplicationException
	{
		public Custom3Exception(String msg) : base(msg)
		{
		}
	}
	class Custom4Exception : ApplicationException
	{
		public Custom4Exception(String msg) : base(msg)
		{
		}
	}
	class CustomException : ApplicationException
	{
		public CustomException(String msg) : base(msg)
		{
		}
	}

	class UnmanagedException
	{
		public UnmanagedException(int err)
		{
			this.err = err;
		}
		public int err;
	}

	/// <summary>
	/// Summary description for BasicExceptions.
	/// </summary>
	// Not the most exciting unhandled exception handler in the world
	class BasicExceptions
	{
		[DllImport("user32.dll")]
		public static extern int MessageBox(int hWnd, string text, string caption, uint type);

		static void SecretMethod()
		{
			// Create a security permission object to describe the
			// UnmanagedCode permission:
			SecurityPermission perm = 
				new SecurityPermission(SecurityPermissionFlag.UnmanagedCode);

			// Deny the UnmanagedCode from our current set of permissions.
			// Any method that is called on this thread until this method 
			// returns will be denied access to unmanaged code.
			perm.Deny();
			MessageBox(0, "Hello from my secret method. Shhh . . .", "Top Secret", 0);
		}
		static void Main(string[] args)
		{
			// Setup unhandled exception handler
			Thread.GetDomain().UnhandledException += 
				new UnhandledExceptionEventHandler(UnhandledExceptionHandler);

			Console.WriteLine("\nDivide by zero\n");

			// Try a divide by zero
			try
			{
				int x = 0;
				x = 1/x;
			}
			catch(DivideByZeroException e)
			{
				// Dump interesting exception information
				Console.WriteLine (e.ToString());
			}

			Console.WriteLine("\nAccess violation\n");

			// Try an access violation
			try
			{
				Object o = null;
				Console.WriteLine(o.ToString());
			}
			catch(NullReferenceException e)
			{
				// Dump interesting exception information
				Console.WriteLine (e.ToString());
			}

			Console.WriteLine("\nStack Overflow\n");
			try
			{
				// Recurse.StackOverflow();
			}
			catch(StackOverflowException e)
			{
				// Dump interesting exception information
				Console.WriteLine (e.ToString());
			}

			Console.WriteLine("\nCustom exception\n");

			// Try custom exception
			try
			{
				throw new CustomException("I am throwing a custom exception");
			}
			catch(CustomException e)
			{
				// Dump interesting exception information
				Console.WriteLine (e.ToString());
			}

			Console.WriteLine("\nObject-based exception\n");

			// Try an object-based exception (InvalidCast)
			try
			{
				int n = 5;
				Object o = n;
				string s = (string)o;
			}
			catch(InvalidCastException e)
			{
				// Dump interesting exception information
				Console.WriteLine (e.ToString());
			}

			Console.WriteLine("\nSecurity Permission\n");
			try
			{
				SecretMethod();
			}
			catch(SecurityException e)
			{
				// Dump interesting exception information
				Console.WriteLine (e.ToString());
			}

			Console.WriteLine("\nFile I/O exceptions\n");
			StreamReader streamReader = null;
			try 
			{
				FileStream stream = File.OpenRead(@"..\..\BasicExceptions.cs");
				streamReader = new StreamReader(stream, Encoding.ASCII);
				streamReader.BaseStream.Seek(0, SeekOrigin.Begin);
				string s = streamReader.ReadToEnd();
				streamReader.BaseStream.Seek(0, SeekOrigin.Begin);
				// Console.WriteLine(s);
			}
			catch(Exception e)
			{
				// Dump interesting exception information
				Console.WriteLine ("There has been an error:\n{0}", e.ToString());
			}
			finally
			{
				Console.WriteLine ("Closing the file");
				if(streamReader != null)
					streamReader.Close();
			}


			Console.WriteLine("\nFiltering exceptions\n");
			try 
			{
				Random r = new Random();
				switch(r.Next(4) + 1)
				{
					case 1:
						throw new Custom1Exception("I am throwing a Custom1Exception");
					case 2:
						throw new Custom2Exception("I am throwing a Custom2Exception");
					case 3:
						throw new Custom3Exception("I am throwing a Custom3Exception");
					case 4:
						throw new Custom4Exception("I am throwing a Custom4Exception");
				}
			}
			catch(Custom1Exception e)
			{
				Console.WriteLine ("There has been an Custom1Exception error.\n{0}", e.ToString());
			}
			catch(Custom2Exception e)
			{
				Console.WriteLine ("There has been an Custom2Exception error.\n{0}", e.ToString());
			}
			catch(Custom3Exception e)
			{
				Console.WriteLine ("There has been an Custom3Exception error.\n{0}", e.ToString());
			}
			catch(Custom4Exception e)
			{
				Console.WriteLine ("There has been an Custom4Exception error.\n{0}", e.ToString());
			}

			// This is bad programming style.
			int sum = 0;
			int i = 0;
			int [] array = new int [] {1,2,3,4,5,6};
			try
			{
				while(true)
				{
					sum += array[i++];
				}
			}
			catch(Exception e)
			{
				Console.WriteLine ("\nArray bounds exception:\n{0}", e);
			}
			Console.WriteLine("\nSum is {0}\n", sum);

			Console.WriteLine("\nDivide by zero\n");

			Console.WriteLine("\nNested exceptions\n");
			try
			{
				// Try a divide by zero
				try
				{
					int x = 0;
					x = 1/x;
				}
				catch(DivideByZeroException e)
				{
					// Dump interesting exception information
					Console.WriteLine (e.ToString());
					throw;
				}
			}
			catch(DivideByZeroException e)
			{
				// Dump interesting exception information
				Console.WriteLine (e.ToString());
			}

			Console.WriteLine("\nUnhandled exception\n");

			// Try to divide by zero and see where it gets you.
			int y = 0;
			y = 1 / y;

			Console.WriteLine("\nThis point is never reached.\n");
		}
		public static void UnhandledExceptionHandler(object sender, UnhandledExceptionEventArgs args)
		{
			Console.WriteLine("Unhandled Exception!");
			Console.WriteLine("Sender: {0}", sender.ToString());
			Console.WriteLine("Type: {0}", args.ExceptionObject.GetType().ToString());         
			Console.WriteLine("IsTerminating: {0}", args.IsTerminating);         
		}
	}
}
