package PerlWordCount::WordCount;
#-----------------------------------------------------------------------

# Create new 'PerlWordCount::WordCount' object, 
# optionally setting 'Text' property

=for interface
     [interface: pure]
     static WordCount WordCount();
     static WordCount WordCount(str text);
=cut

sub new {
    my ($package,$text) = @_;
    my $self = bless {} => $package;
    $self->Text($text);
    return $self;
}
#-----------------------------------------------------------------------

# Get or set 'Text' property and update word counts

=for interface
     str Text;
=cut

sub Text {
    my ($self,$text) = @_;
    return $self->{Text} unless defined $text;

    $self->{Text} = $text;
    $self->{Count} = {};

    # convert all non-alphas to single space
    $text =~ tr/a-zA-Z/ /cs;

    # split lowercase text on single space and count words
    ++$self->{Count}->{$_} foreach split(' ', lc $text);
}
#-----------------------------------------------------------------------

# 'Count' returns number of occurances or throws an exception

=for interface
     int Count();
     int Count(str word);
=cut

sub Count {
    my ($self,$word) = @_;

    # Without arguments: return number of distinct words
    return int keys %{$self->{Count}} unless defined $word;

    # Throw exception if $word does not exist in 'Text'
    die "Word '$word' not found" unless defined $self->{Count}->{$word};

    # Return number of occurances
    return $self->{Count}->{$word};
}
#-----------------------------------------------------------------------

# 'Words' returns alphabetically sorted *list* of all words in 'Text'

=for interface
     wantarray! str[] Words();
=cut

sub Words {
    my ($self) = @_;
    return sort keys %{$self->{Count}};
}

1;
