//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Threading;

namespace ThreadExceptions
{
	public delegate int WorkerDelegate();

	public class ExceptionHandlers 
	{
		static void ExceptionFilter(object o, UnhandledExceptionEventArgs e) 
		{
			Console.WriteLine("Inside ExceptionFilter on thread: {0} IsTerminating: {1}",
				AppDomain.GetCurrentThreadId(), e.IsTerminating);
		}
		static int WorkerThatThrowsException() 
		{
			Console.WriteLine("Inside WorkerThatThrowsException on thread {0}", AppDomain.GetCurrentThreadId());
			throw new ApplicationException("Exception raised in WorkerThatThrowsException");
			Console.WriteLine("Inside WorkerThatThrowsException about to return the answer");
			return 0;
		}
		static void EndInvokeCallback(IAsyncResult iar) 
		{
			Console.WriteLine("Inside EndInvokeCallback on thread {0}", AppDomain.GetCurrentThreadId());
			WorkerDelegate wd = (WorkerDelegate)iar.AsyncState;
			try 
			{
				int result = wd.EndInvoke(iar);
				Console.WriteLine("Inside EndInvokeCallback the answer was {0}", result);
			} 
			catch (Exception e) 
			{
				Console.WriteLine("Caught exception in EndInvokeCallback:");
				Console.WriteLine(e.ToString());
			}
		}
		static void UnprotectedEndInvokeCallback(IAsyncResult iar) 
		{
			Console.WriteLine("Inside UnprotectedEndInvokeCallback on thread {0}", AppDomain.GetCurrentThreadId());
			WorkerDelegate wd = (WorkerDelegate)iar.AsyncState;
			int result = wd.EndInvoke(iar);
			Console.WriteLine("Inside EndInvokeCallback the answer was {0}", result);
		}
		static void Main() 
		{
			Console.WriteLine("Inside Main on thread {0}",
				AppDomain.GetCurrentThreadId());
			AppDomain.CurrentDomain.UnhandledException += new
				UnhandledExceptionEventHandler(ExceptionFilter);

			WorkerDelegate wd = new WorkerDelegate(WorkerThatThrowsException);
			wd.BeginInvoke(new AsyncCallback(UnprotectedEndInvokeCallback), wd);
			Console.ReadLine();
			wd.BeginInvoke(new AsyncCallback(EndInvokeCallback), wd);
			Console.ReadLine();
			Console.WriteLine();
			try 
			{
				int result = wd();
				Console.WriteLine("Inside Main the answer was {0}", result);
			} 
			catch (Exception e) 
			{
				Console.WriteLine("Caught exception in Main:");
				Console.WriteLine(e.ToString());
			}
		}
	}
}
